/* GLOBAL VARIABLES */
// __tailsContent points to the ... content in sidebar. Each MFO is a row in this table. 
var __tailsContent = null;

// __tailsMFO is the global array for all MFOs in the sidebar.
var __tailsMFO = new Array();

// __tailsSource is the global source URL of the page that the MFOs were in
var __tailsSource = '';

/* EVENT LISTENERS */

/* 
 * On load of sidebar some intialisations are done.
 */
function processOnLoad() 
{
  /* Add page load event listener and tab changed event listener. 
     When page is loaded or tab changes sidebar is rebuilt. 
   */
  window.top.getBrowser().addEventListener("load", onPageShow, true);
  window.top.getBrowser().addEventListener("pageshow", onPageShow, true);
  window.top.getBrowser().tabContainer.addEventListener("select", onTabChanged, true);
  
  /* global variable content points to the insertionpoint within the sidebar */
  __tailsContent = $_("tailsexportSidebar").getElementsByTagName("vbox").item(1);
  __tailsMFO = new Array();

  /* Ensure sidebar is filled on loading */
  onTabChanged();
}

/* 
 * On unload of sidebar some cleanup is done.
 */
function processOnUnload() 
{
  /* When unloading the sidebar remove the event listeners */
  window.top.getBrowser().removeEventListener("load", onPageShow, true);
  window.top.getBrowser().removeEventListener("pageshow", onPageShow, true);
  window.top.getBrowser().tabContainer.removeEventListener("select", onTabChanged, true);

  clearSidebar();

  /* free global variable memory */
  __tailsContent = null;
  __tailsMFO = null;
}

/* 
 * On page show
 */
function onPageShow(event) 
{
  if (event.originalTarget instanceof HTMLDocument)
  {
    if (!event.originalTarget.defaultView.frameElement
        && isCurrentTab(getTabForDocument(event.originalTarget)))
    {
      clearSidebar();

      processMFOs(event.originalTarget);
    }
  }
}

/*
 * When user moves to another tab. And this tab has MFOs.
 * __tailsContainsMicroformats filled in tails.js.
 */
function onTabChanged(event) 
{
  clearSidebar();

  var tab = window.top.getBrowser().selectedTab.linkedBrowser;

  if (tab && tab.__tailsContainsMicroformats)
  {
    processMFOs(tab.contentDocument);
  }
}

/* Remove all contents of the sidebar */
function clearSidebar() 
{
  /* Remove mfo entries on sidebar */
  while (__tailsContent.hasChildNodes && __tailsContent.firstChild)
    __tailsContent.removeChild(__tailsContent.firstChild);

  /* Reset mfo array */
  __tailsMFO = new Array();
}

/* FUNCTIONS */
/*
 * Get all formats and check if present. If so process them.
 */
function processMFOs(doc) 
{
  var i, j, re, MFOs;

  var formats = getAllFormats();

  __tailsSource = doc.location.toString();

  for (j=0; j<formats.length; j++) 
  {
    if( getDefault(formats[j].className) )
    {
      MFOs = document.evaluate("//*[contains(@"+formats[j].classAttribute+", '"+formats[j].classValue+"')]", doc, null, XPathResult.UNORDERED_NODE_SNAPSHOT_TYPE, null);
  
      re = new RegExp("\\b"+formats[j].classValue+"\\b");

      // ensure nodes are not just substring matches
      for (i=0; i<MFOs.snapshotLength; i++) 
      {
        if (MFOs.snapshotItem(i).getAttribute(formats[j].classAttribute).match(re)) { processFormat(formats[j], MFOs.snapshotItem(i) ); }
      }
    }
  }
}

/*
 * Build up an object to be shown in the sidebar.
 *
 * Check if object build should be added (determined in format itself). If so 
 * then add it to the sidebar and to the global mfo array.
 * 
 * Add export button if exportable object.
 */
function processFormat(format, node) 
{
  var doc, menu;

  var formatObj = {};

  format.processChildNode(node, formatObj);
  formatObj = buildFormatObject(format, node, formatObj);

  formatObj.className = format.className;

  if (format.add(formatObj)) 
  {
    // xml points to node, it isn't a copy!
    formatObj.xml = node;

    // load the xul file of the format and populate it
    doc = document.implementation.createDocument("","",null);
    doc.async = false;
    doc.load(format.xulFile);

    format.populate(formatObj, doc.documentElement, doc);

    menu = getElementByClassName(doc, "hbox", "tails_mfo_buttons");

    if( getDefault("showSourceHTML") )
    {
      // add show button
      menu.appendChild(createShowButton());
    }

    // add export button and save essentials
    if(format.mfoexport && getDefault("exportFormats") )
    {
      formatObj.xsl = format.xslFile;
      formatObj.ext = format.mfoextension;

      menu.appendChild(createExportButton());
    }

    // add bluetooth button and save essentials
    if( window.navigator.platform.substring(0,3) == 'Mac' && format.mfoexport && getDefault("exportFormats") )
    {
      formatObj.xsl = format.xslFile;
      formatObj.ext = format.mfoextension;

      menu.appendChild(createBluetoothButton());
    }

    // add export button and save essentials
    if( format.mfogeo && getDefault("showLocation") && format.mfogeo(formatObj) == true)
    {
      menu.appendChild(createGeoButton());
    }

    // add to sidebar
    __tailsContent.appendChild(doc.documentElement);

    // add to global var.
    __tailsMFO[__tailsMFO.length] = formatObj;
  }
}

/*
 * Build the object. Recursive foreach node.
 */
function buildFormatObject(format, node, formatObj)
{
  var i, child;

  var children = node.childNodes;

  for (i=0; i<children.length; i++) 
  {
    child = children.item(i);

    if (child.hasChildNodes && child.tagName)
    {
      format.processChildNode(child, formatObj);
      buildFormatObject(format, child, formatObj);
    }
  }
  return formatObj;
}

/*
 * Generate a show button.
 */
function createShowButton()
{
  var strbundle=document.getElementById("strings");

  var item = document.createElement("label");

  item.setAttribute("value", strbundle.getString("show_button_label"));
  item.setAttribute("class", "tails_mfo_show_button");
  item.setAttribute("onclick", "showMFO("+__tailsMFO.length+");");

  return item;
}

/*
 * Generate a bluetooth button.
 */
function createBluetoothButton()
{
  var strbundle=document.getElementById("strings");

  var item = document.createElement("label");

  item.setAttribute("value", strbundle.getString("bluetooth_button_label"));
  item.setAttribute("class", "tails_mfo_export_button");
  item.setAttribute("onclick", "exportMFO("+__tailsMFO.length+", true );");

  return item;
}

/*
 * Generate an export button.
 */
function createExportButton()
{
  var strbundle=document.getElementById("strings");

  var item = document.createElement("label");

  item.setAttribute("value", strbundle.getString("export_button_label"));
  item.setAttribute("class", "tails_mfo_export_button");
  item.setAttribute("onclick", "exportMFO("+__tailsMFO.length+");");

  return item;
}

function createGeoButton()
{
  var strbundle=document.getElementById("strings");

  var item = document.createElement("label");

  item.setAttribute("value", strbundle.getString("geo_button_label"));
  item.setAttribute("class", "tails_mfo_geo_button");
  item.setAttribute("onclick", "geoMFO("+__tailsMFO.length+");");

  return item;
}

